@@ -31,6 +31,8 @@ module Data.Yaml
     , encodeFile
     , decode
     , decodeFile
+      -- ** Better error information
+    , decodeEither
     ) where
 
 import qualified Text.Libyaml as Y
@@ -127,7 +129,7 @@ instance MonadTrans PErrorT where
 instance MonadIO m => MonadIO (PErrorT m) where
     liftIO = lift . liftIO
 
-type Parse = StateT (Map.Map String Value) IO
+type Parse = StateT (Map.Map String Value) (C.ResourceT IO)
 
 requireEvent :: Event -> C.Sink Event Parse ()
 requireEvent e = do
@@ -145,21 +147,24 @@ parse = do
     requireEvent EventStreamEnd
     return res
 
-parseScalar :: ByteString -> Anchor
+parseScalar :: ByteString -> Anchor -> Style
             -> C.Sink Event Parse Text
-parseScalar v a = do
+parseScalar v a style = do
     let res = decodeUtf8With lenientDecode v
     case a of
         Nothing -> return res
         Just an -> do
-            lift $ modify (Map.insert an $ textToValue res)
+            lift $ modify (Map.insert an $ textToValue style res)
             return res
 
-textToValue :: Text -> Value -- FIXME check for quoting style?
-textToValue "true" = Bool True
-textToValue "false" = Bool False
-textToValue "null" = Null
-textToValue t
+textToValue :: Style -> Text -> Value
+textToValue SingleQuoted t = String t
+textToValue DoubleQuoted t = String t
+textToValue Folded t = String t
+textToValue _ "true" = Bool True
+textToValue _ "false" = Bool False
+textToValue _ "null" = Null
+textToValue _ t
     | Right (x, "") <- signed decimal t = Number $ I x
     | Right (x, "") <- double t = Number $ D x
     | otherwise = String t
@@ -168,7 +173,7 @@ parseO :: C.Sink Event Parse Value
 parseO = do
     me <- CL.head
     case me of
-        Just (EventScalar v _t _s a) -> fmap textToValue $ parseScalar v a
+        Just (EventScalar v _t style a) -> fmap (textToValue style) $ parseScalar v a style
         Just (EventSequenceStart a) -> parseS a id
         Just (EventMappingStart a) -> parseM a M.empty
         Just (EventAlias an) -> do
@@ -213,7 +218,7 @@ parseM a front = do
         _ -> do
             CL.drop 1
             s <- case me of
-                    Just (EventScalar v _ _ a') -> parseScalar v a'
+                    Just (EventScalar v _ style a') -> parseScalar v a' style
                     _ -> liftIO $ throwIO $ UnexpectedEvent me Nothing
             o <- parseO
 
@@ -231,24 +236,31 @@ parseM a front = do
 decode :: FromJSON a
        => ByteString
        -> Maybe a
-decode bs = unsafePerformIO $ fmap (either (const Nothing) id) $ decodeHelper (Y.decode bs)
+decode bs = unsafePerformIO
+          $ fmap (either (const Nothing) (either (const Nothing) Just))
+          $ decodeHelper (Y.decode bs)
 
 decodeFile :: FromJSON a
            => FilePath
            -> IO (Maybe a)
-decodeFile fp = decodeHelper (Y.decodeFile fp) >>= either throwIO return
+decodeFile fp = decodeHelper (Y.decodeFile fp) >>= either throwIO (return . either (const Nothing) id)
+
+decodeEither :: FromJSON a => ByteString -> Either String a
+decodeEither bs = unsafePerformIO
+                $ fmap (either (Left . show) id)
+                $ decodeHelper (Y.decode bs)
 
 decodeHelper :: FromJSON a
              => C.Source Parse Y.Event
-             -> IO (Either ParseException (Maybe a))
+             -> IO (Either ParseException (Either String a))
 decodeHelper src = do
-    x <- try $ flip evalStateT Map.empty $ C.runResourceT $ src C.$$ parse
+    x <- try $ C.runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
     case x of
         Left e
             | Just pe <- fromException e -> return $ Left pe
             | Just ye <- fromException e -> return $ Left $ InvalidYaml $ Just $ show (ye :: YamlException)
             | otherwise -> throwIO e
-        Right y -> return $ Right $ parseMaybe parseJSON y
+        Right y -> return $ Right $ parseEither parseJSON y
 
 array :: [Value] -> Value
 array = Array . V.fromList
