@@ -135,7 +135,7 @@ int yaml_parser_set_input_filename(yaml_parser_t *parser, const char *filename)
 	yaml_parser_set_input_file(parser, in);
 }
 
-int fclose_helper(FILE *file, yaml_parser_t *parser)
+int fclose_helper(FILE *file)
 {
 	if (! file) return 0;
 	return fclose(file);
